void TFoundFactor::output(std::ostream &ostr) const
{
  ostr << x;
  if (e!=1) ostr << "^" << e;
  if (comment!="") ostr << " " << comment;
  else
   {
     if (!mpz_probab_prime_p(x,probab_prime_checks)) ostr << "[composite]";
   }
}

void TFoundFactors::AutoSave() const
{
  std::ofstream ofstr(FoundFactorsFile.c_str());
  ofstr << regarding << endl;
  Save(ofstr);
}

void TFoundFactors::AutoLoad()
{
#ifdef VERBOSE_INFO
  cout << "Autoload FoundFactors" << endl;
#endif
  std::ifstream ifstr(FoundFactorsFile.c_str());
  char buf [32768];
  ifstr.getline(buf,sizeof(buf),'\n'); regarding=buf;
  Load(ifstr);
}


const char* const _MAL(void)
{
  // returns empty string on first call, and  " * " otherwise.
  // Task: Prevent costly checks for suppressing the trivial factor 1
  // as the fist resp. last factor.

#ifdef NOTIFY_PARENT
  // send SIGUSR2 to the parent to notify that a new
  // factor has found
  kill(getppid(),SIGUSR2); // send SIGUSR2 to the parent process
#endif

  static bool first_call = true;
  if (first_call)
   {
     first_call=false;
     return "";
   }
  else return " * ";
}

std::string MAL(const mpz_t factor, const unsigned int exponent=1)
{
  FoundFactors.insert(TFoundFactor(factor,exponent));
  FoundFactors.AutoSave();
  ostringstream helper;
  helper << _MAL() << factor;
  if (exponent>1) helper << "^" << exponent;
  return helper.str();
}

std::string MAL(const mpz_t factor, const unsigned int exponent, const std::ostringstream &comment)
{
  FoundFactors.insert(TFoundFactor(factor,exponent,comment.str()));
  FoundFactors.AutoSave();
  ostringstream helper;
  helper << _MAL() << factor;
  if (exponent>1) helper << "^" << exponent;
  return helper.str()+comment.str();
}


std::string MAL(const mpz_t factor, const std::ostringstream &comment)
{
  FoundFactors.insert(TFoundFactor(factor,1,comment.str()));
  FoundFactors.AutoSave();
  ostringstream helper;
  helper << _MAL() << factor;
  return helper.str()+comment.str();
}

std::string MAL(const long unsigned int factor, const unsigned int exponent=1)
{
  FoundFactors.insert(TFoundFactor(factor,exponent));
  FoundFactors.AutoSave();
  ostringstream helper;
  helper << _MAL() << factor;
  if (exponent>1) helper << "^" << exponent;
  return helper.str();
}
